package com.blackhillsoftware.samples.primeshifttopjobs;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.time.*;

import com.blackhillsoftware.smf.SmfRecord;
import com.blackhillsoftware.smf.SmfRecordReader;
import com.blackhillsoftware.smf.Utils;
import com.blackhillsoftware.smf.smf30.Smf30Record;
import com.blackhillsoftware.smf.smf30.ProcessorAccountingSection;

public class App 
{
	
	public static void main(String[] args) throws IOException 
	{			
		
		LocalTime primeStart = LocalTime.of(8, 30);
		LocalTime primeEnd = LocalTime.of(17, 30);
		
		// We need a Jobname->JobData map for each day of the week.
		// So we will create a List (array) of HashMap<String, JobData>.
		
		List<HashMap<String, JobData>> jobsByDay = 
				new ArrayList<HashMap<String, JobData>>();
		
		// Populate the list for each day of the week
		// We need entries for days 1-7. Entry 0 is unused
		for (int i=0; i <= 7; i++) 
		{ 
			jobsByDay.add(new HashMap<String, JobData>());
		}
		
		// Read and process the data
		
		SmfRecordReader reader;
		if (args.length == 0)
		{
			reader = SmfRecordReader.fromDD("INPUT");
		}
		else
		{
			reader = SmfRecordReader
					.fromStream(new FileInputStream(args[0]));
		}
		
		for (SmfRecord record : reader) 
		{
			// We are only interested in type 30 subtype 2 or 3 
			if (record.recordType() == 30 
					&& (record.subType() == 2 || record.subType() == 3)) 
			{
				
				DayOfWeek recordDayOfWeek = record.smfDate().getDayOfWeek();
				LocalTime recordTime = record.smfTime();
				
				// check if SMF record was created during prime shift Monday-Friday
				if (recordDayOfWeek.getValue() >= DayOfWeek.MONDAY.getValue()
						&& recordDayOfWeek.getValue() <= DayOfWeek.FRIDAY.getValue()
						&& recordTime.isAfter(primeStart) 
						&& recordTime.isBefore(primeEnd)) 
				{
					
					// Create a type 30 record from the original record
					Smf30Record r30 = new Smf30Record(record);
					
					// Iterate over the processor accounting sections
					// Since there is either 0 or 1 section in a record, this is
					// just a convenient way to skip records without the section
	        		for (ProcessorAccountingSection proc : 
	        			r30.processorAccountingSections())
	        		{
	        			// get collection of jobs for this day
	        			Map<String, JobData> day = jobsByDay.get(recordDayOfWeek.getValue());
	        			
	        			String jobname = r30.identificationSection().smf30jbn();
	        			JobData jobentry = day.get(jobname);
		        		if (jobentry == null) // nothing for this jobname, create new
		        		{
		        			day.put(jobname, new JobData(proc));
		        		}
		        		else
		        		{
		        			jobentry.add(proc);
		        		}
	        		}
	        	}
        	}
        };
        reader.close();
        writeReport(jobsByDay);     
	}

	private static void writeReport(
			List<HashMap<String, JobData>> dailyJobs) {
		       
		for (DayOfWeek day : DayOfWeek.values()) 
        {
        	
        	// get the jobs for the day as a List so we can sort them
        	List<Entry<String, JobData>> jobs = 
        			new ArrayList<Entry<String, JobData>>(dailyJobs.get(day.getValue()).entrySet());
        	
        	if (jobs.size() > 0) // if we have data for this day
        	{
        		// sort list by CPU time descending
	            Collections.sort(jobs, 
	            		new Comparator<Entry<String, JobData>>()
	            		{
			                public int compare(
			                		Entry<String, JobData> s1, 
			                		Entry<String, JobData> s2) 
			                {
			                    return s2.getValue().cpTime.compareTo(s1.getValue().cpTime);
			                }
	            		}
	            );
	            
	            // calculate total CPU for the day
	            Duration totalCp = Duration.ZERO;
	            for (Entry<String,JobData> entry : jobs)
	            {
	        		totalCp = totalCp.plus(entry.getValue().cpTime);        		            	
	            }
	        	
	        	// write output for top 10 jobs
	            
	            // Headings
	        	System.out.format("%n%s%n", 
	        			day.toString());
	        	System.out.format("%-8s %11s %5s %11s %11s %n",
	        			"Name",
	        			"CPU",
	        			"CPU%", 
	        			"zIIP", 
	        			"zAAP");
	        	
	        	int count = 0;
	            for (Entry<String,JobData> entry : jobs)
	            {
	        		JobData jobinfo = entry.getValue();
	        		
	        		// write detail line

	        		System.out.format("%-8s %11s %4.0f%% %11s %11s %n",
	        				entry.getKey(), // jobname
	        				hhhmmss(jobinfo.cpTime),
	        				Utils.divide(jobinfo.cpTime,  totalCp) * 100,
	        				hhhmmss(jobinfo.ziipTime),
	        				hhhmmss(jobinfo.zaapTime));
	        		
	        		// limit to top 10 jobs
	        		if (++count >= 10) break;
	            }
        	}
        }
	}
	
	private static String hhhmmss(Duration dur)
	{
		long hours = dur.toHours();
		long minutes = dur
				.minus(Duration.ofHours(hours))
				.toMinutes();
		long seconds = dur
				.minus(Duration.ofHours(hours))
				.minus(Duration.ofMinutes(minutes))
				.toMillis() / 1000;
		return String.format("%d:%02d:%02d", hours, minutes, seconds); 
	}
	
	private static class JobData {
		JobData(ProcessorAccountingSection proc)
		{
			add(proc);
		}
		
		public void add(ProcessorAccountingSection proc)
		{
			cpTime = cpTime 
					.plus(proc.smf30cpt())
					.plus(proc.smf30cps())
					.minus(proc.smf30TimeOnIfa()) 
					.minus(proc.smf30TimeOnZiip());
			zaapTime = zaapTime.plus(proc.smf30TimeOnIfa());
			ziipTime = ziipTime.plus(proc.smf30TimeOnZiip());
		}
		
		Duration cpTime = Duration.ZERO;
		Duration ziipTime = Duration.ZERO;
		Duration zaapTime = Duration.ZERO;
	}
	
}
