package com.blackhillsoftware.samples.recordcount;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.blackhillsoftware.smf.SmfRecord;
import com.blackhillsoftware.smf.SmfRecordReader;

public class App 
{
    public static void main( String[] args ) throws IOException
    {
    	// use a HashMap to keep the statistics
		Map<Integer, RecordStats> statsMap = 
				new HashMap<Integer, RecordStats>();

		// Read from a DD or file
		SmfRecordReader reader;
		if (args.length == 0)
		{
			reader = SmfRecordReader.fromDD("INPUT");
		}
		else
		{
			reader = SmfRecordReader
					.fromStream(new FileInputStream(args[0]));
		}

		// SmfRecordReader implements Iterable<SmfRecord>, so we can 
		// simply iterate to read SmfRecords
		for (SmfRecord record : reader) 
		{		
			Integer key = (record.recordType() << 16) 
					+ (record.hasSubtypes() ? record.subType() : 0);
			
			// try to retrieve an existing entry
			RecordStats stats = statsMap.get(key);
			
			if (stats == null) // none -> create new
			{
				statsMap.put(key, new RecordStats(record));
			}
			else // otherwise add this record to existing
			{
				stats.add(record);
			}
		};

		reader.close();

		writeReport(statsMap);
    }

    
	private static void writeReport(Map<Integer, RecordStats> statsMap) 
	{
		// Turn the HashMap into a list so it can be sorted
		List<Entry<Integer, RecordStats>> results = 
				new ArrayList<Entry<Integer, RecordStats>>(statsMap.entrySet());
		
		// get the total bytes from all record types
		long totalbytes = 0;
		for (Entry<Integer, RecordStats> entry : results)
		{
			totalbytes += entry.getValue().bytes;
		}
		
		// sort by total bytes descending
        Collections.sort(results, 
        		new Comparator<Entry<Integer, RecordStats>>()
        		{
	                public int compare(
	                		Entry<Integer, RecordStats> s1, 
	                		Entry<Integer, RecordStats> s2) 
	                {
	                    return Long.compare( // reversed to sort descending 
	                    				s2.getValue().bytes, 
	                    				s1.getValue().bytes);
	                }
        		}
        		);
               
        // write heading
		System.out.format("%5s %8s %11s %11s %7s %9s %9s %9s %n",
				"Type",
				"Subtype",
				"Records",
				"MB",
				"Pct",
				"Min",
				"Max",
				"Avg");
		
		// write data 
		for (Entry<Integer, RecordStats> entry : results)
		{
    		System.out.format("%5d %8d %11d %11d %7.1f %9d %9d %9d %n",
    				entry.getKey().intValue() >> 16,
    				entry.getKey().intValue() & 0xFFFF,
    				entry.getValue().count,
    				entry.getValue().bytes / (1024*1024),
    				(float)(entry.getValue().bytes) / totalbytes * 100,
    				entry.getValue().min,
    				entry.getValue().max,
    				entry.getValue().bytes / entry.getValue().count);
		}
	}

	/**
	 * Statistics for a type/subtype combination
	 */
	private static class RecordStats
	{
		public RecordStats(SmfRecord record)
		{
			count = 1;
			bytes = min = max = record.recordLength();
		}	
		
		int count;
		long bytes;
		int max;
		int min;

		public void add(SmfRecord record) 
		{
			count++;
			int length = record.recordLength();
			bytes += length;
			if (length < min)
				min = length;
			if (length > max)
				max = length;
		}
	}
}
